from tdf_tools.tools.shell_dir import ProjectType, ShellDir
from tdf_tools.tools.translate.ios.ios_translate import iOSTranslate
from tdf_tools.tools.translate.translate_lint import TranslateLint
from tdf_tools.tools.translate.flutter.flutter_translate_tools import (
    FlutterTranslateTools,
)


class Translate:
    """
    国际化相关：tl translate -h 查看详情
    """

    def __init__(self) -> None:
        self.lint = TranslateLint()

    def start(self):
        """
        国际化相关：通过交互式的方式处理国际化
        """
        ShellDir.dirInvalidate()
        projectType = ShellDir.getProjectType()
        if projectType == ProjectType.FLUTTER:
            FlutterTranslateTools().translate()
        elif projectType == ProjectType.IOS:
            iOSTranslate().translate()

    def module(self, name):
        """
        国际化相关：指定模块进行国际化
        """
        ShellDir.dirInvalidate()
        projectType = ShellDir.getProjectType()
        if projectType == ProjectType.FLUTTER:
            FlutterTranslateTools().translate_module(name)
        elif projectType == ProjectType.IOS:
            iOSTranslate().translate_module(name)
        exit(0)

    def integrate(self):
        """
        国际化相关：整合所有组件的国际化文件到一个文件中，用来维护一份国际化文件
        """
        ShellDir.dirInvalidate()
        projectType = ShellDir.getProjectType()
        if projectType == ProjectType.FLUTTER:
            pass
        elif projectType == ProjectType.IOS:
            iOSTranslate().integrate()
        exit(0)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-


from tdf_tools.tools.cmd import Cmd
from tdf_tools.tools.gitlab.python_gitlab_api import GitlabAPI
from tdf_tools.tools.print import Print


class GitlabUtils(object):
    def __init__(self):
        Print.stage("gitlab管理")

    def diff(self, targetBranch="master"):
        res = Cmd.run(
            "git diff --name-only {0}..{1}".format(self._getCurBranch(), targetBranch)
        ).splitlines()
        if len(res) > 0:
            Print.str("{0} files diff".format(len(res)))

    def commit(self, message):
        Cmd.runAndPrint("git add .")
        Cmd.runAndPrint("git commit -m {0}".format(message))

    def status(self):
        res = Cmd.run("git status -s")
        if len(res.splitlines()) > 0:
            Print.str(res)
        else:
            Print.str("无文件要提交，干净的工作区")

    def unChangeValidate(self):
        res = Cmd.run("git status -s")
        if len(res.splitlines()) > 0:
            Print.str("本地仍有修改，请先提交代码")
            Print.str(res)
        else:
            exit(1)

    def clone(self, module, gitUrl):
        # 这边给clone下的项目重命名一下，避免仓库名和模块名不一致
        res = Cmd.runAndPrint("git clone {0} {1}".format(gitUrl, module))
        Print.str(res)

    def merge(self, source_branch):
        cmd = "git merge {0}".format(source_branch)
        state = Cmd.system(cmd)
        GitlabUtils.checkStateWithMsg(state, cmd)

    # 一个模块创建MR
    # def mergeRequestCreateForOneModule(self, module):
    # api = GitlabAPI()
    # printStr("模块{0}创建MR：from <{1}> into <{2}>".format(
    #     module, self.initJsonData['featureBranch'], self.initJsonData['testBranch']))
    # api.createMR(
    #     self.moduleJsonData[module]['id'], self.initJsonData['featureBranch'],
    # self.initJsonData['testBranch'])

    # 为所有模块创建MR

    def mergeRequestCreate(self, module, sourceBranch, targetBranch):
        api = GitlabAPI()
        GitlabUtils.unChangeValidate()
        Print.str(
            "模块{0}创建MR：from <{1}> into <{2}>".format(module, sourceBranch, targetBranch)
        )
        api.createMR(self.moduleJsonData[module]["id"], sourceBranch, targetBranch)
        # 命令形式提交mr
        # for module in moduleNameList:
        #     moduleDir = os.path.join(projectModuleDir, module)
        #     os.chdir(moduleDir)
        #     printTitle(module)

        #     # 确保本地代码都应提交了，以免出现代码有忘记提交的，导致提测代码不是最新的
        #     GitlabUtils.unChangeValidate()

        #     cmd = 'git push -o merge_request.create -o merge_request.source={0} -o
        #     merge_request.target={1} -o merge_request.assignee_id={2}'.format(
        #         initJsonData['featureBranch'], initJsonData['testBranch'], "819")
        #     state = os.system(cmd)
        #     GitlabUtils.checkStateWithMsg(state, cmd)
        # print()

    def _getCurBranch(self):
        return GitlabUtils._runCmd("git rev-parse --abbrev-ref HEAD").replace("\n", "")

    def fetch(self):
        cmd = "git fetch"
        state = Cmd.system(cmd)
        GitlabUtils.checkStateWithMsg(state, cmd)

    def pull(self):
        cmd = "git pull"
        state = Cmd.system(cmd)
        GitlabUtils.checkStateWithMsg(state, cmd)

    def checkout(self, featureBranch):
        cmd = "git checkout {0}".format(featureBranch)
        state = Cmd.system(cmd)
        GitlabUtils.checkStateWithMsg(state, cmd)

    def createAndCheckout(self, featureBranch, shouldPush=False):
        cmd = "git checkout -b {0}".format(featureBranch)
        state = Cmd.system(cmd)
        GitlabUtils.checkStateWithMsg(state, cmd)

        if shouldPush:
            cmd = "git push --set-upstream origin {0}".format(featureBranch)
            state = Cmd.system(cmd)
            GitlabUtils.checkStateWithMsg(state, cmd)

    # def _branchJudge(self):
    #     isSame = True
    #     curBranch = ''
    #     for module in self.moduleNameList:
    #         moduleDir = os.path.join(projectModuleDir, module)
    #         os.chdir(moduleDir)
    #         cmd = f'git branch'
    #         branchList = GitlabUtils._runCmd(cmd).splitlines()
    #         for branch in branchList:
    #             if branch.startswith('*'):
    #                 if curBranch == '':
    #                     curBranch = branch.split(' ')[1]
    #                 elif curBranch != branch.split(' ')[1]:
    #                     isSame = False
    #     return isSame

    def push(self):
        pushCmd = "git push"
        state = Cmd.system(pushCmd)
        GitlabUtils.checkStateWithMsg(state, pushCmd)

    def checkStateWithMsg(state, cmd):
        if state != 0:
            Print.error(f"execute '{cmd}' failed")

    def executeRawCommand(self, rawCmd):
        Print.str(Cmd.run(rawCmd))



import json
import os
from tdf_tools.tools.cmd import Cmd
from tdf_tools.tools.shell_dir import ShellDir
from tdf_tools.tools.cmd import Cmd
import json
from tdf_tools.tools.print import Print
from tdf_tools.tools.translate.ios.tools.file_util import FileUtil


class BatchPodModel:
    def __init__(self, name: str, branch: str, path: str):
        self.name = name
        self.branch = branch
        self.path = path
        self.source_files: set[str] = set()


class BatchPodTools:
    # 可以进行国际化的列表
    def batchPodList() -> list[BatchPodModel]:
        os.environ["FLUTTER_IS_LOCAL"] = "false"
        os.environ["BATCH_IS_LOCAL"] = "true"
        podfile_json_str: str = Cmd.run("bundle exec pod ipc podfile-json Podfile")
        podfile_json = json.loads(podfile_json_str)
        batch_pod_local = podfile_json["batch_pod_local"]
        pod_models = []
        for pod in batch_pod_local:
            pod_dic = batch_pod_local[pod][0]
            pod_model = BatchPodModel(
                pod,
                pod_dic["branch"],
                pod_dic["path"],
            )
            pod_models.append(pod_model)
        return pod_models

    # 获取 pod 的source_files
    def generate_pod_source_files(pods: list[BatchPodModel]):
        for pod in pods:
            podspec_path = ShellDir.findPodspec(pod.path)
            if isinstance(podspec_path, str):
                pod.source_files = BatchPodTools.get_source_files(podspec_path)
            else:
                Print.error(pod.path + "目录下没找到 podspec 文件")

    # path 为 podspec 所在的路径
    def get_source_files(podspec_path: str) -> set[str]:
        podspec_name = podspec_path.split("/")[-1]
        root_path = podspec_path.split("/" + podspec_name)[0]
        podspec_json_str: str = Cmd.run("pod ipc spec " + podspec_path)
        podspec_json: dict = json.loads(podspec_json_str)
        # 处理 root source_files
        root_source_paths: list[str] = podspec_json.get("source_files")
        if root_source_paths is None:
            root_source_paths = []
        elif isinstance(root_source_paths, str):
            root_source_paths = [root_source_paths]

        # 处理 sub source_files
        subspecs: list[dict] = podspec_json.get("subspecs")
        if subspecs is None:
            subspecs = []
        for subspec in subspecs:
            sub_source_files = subspec.get("source_files")
            if isinstance(sub_source_files, str):
                root_source_paths.append(sub_source_files)
            elif isinstance(sub_source_files, list):
                root_source_paths.extend(sub_source_files)

        # 获取 file_list
        root_source_paths_set: set[str] = set(root_source_paths)
        rb_file = FileUtil.generate_get_files_rb()
        ruby_cmd = ["ruby", rb_file]
        for source_path in root_source_paths_set:
            source_path = root_path + "/" + source_path
            ruby_cmd.append(source_path)
        file_list_str: str = Cmd.run(ruby_cmd, shell=False)
        file_list = file_list_str.split("\n")

        new_file_list: set[str] = set()
        for file in file_list:
            if os.path.isfile(file):
                new_file_list.add(file)

        return new_file_list



import json
from tdf_tools.tools.print import Print
from tdf_tools.tools.shell_dir import ShellDir


class ModuleTools:
    # 获取项目初始化数据
    def getInitJsonData():
        ShellDir.goTdfCacheDir()
        with open("initial_config.json", "r", encoding="utf-8") as readF:
            fileData = readF.read()
            readF.close()
            return json.loads(fileData)

    def getModuleNameList():
        initJsonData = ModuleTools.getInitJsonData()
        if initJsonData.__contains__("moduleNameList") and isinstance(
            initJsonData["moduleNameList"], list
        ):
            moduleNameList = initJsonData["moduleNameList"]
            return moduleNameList
        else:
            Print.error("❌ 请配置moduleNameList的值,以数组形式")

    def getModuleJsonData():  # 获取模块 git相关配置信息
        ShellDir.goTdfCacheDir()
        with open("module_config.json", "r", encoding="utf-8") as readF:
            fileData = readF.read()
            readF.close()
            return json.loads(fileData)



import os
from tdf_tools.tools.translate.ios.tools.batch_pod_tools import (
    BatchPodTools,
)
from tdf_tools.tools.cmd import Cmd
from tdf_tools.tools.fix_header.fix_header_lint_tool import FixHeaderLintTool
from tdf_tools.tools.fix_header.fix_header_replace_tool import FixHeaderReplaceTool
from tdf_tools.tools.shell_dir import ShellDir
from tdf_tools.tools.print import Print
from enum import Enum


class FixHeaderType(Enum):
    REPLACE = 1
    LINT = 2


class FixHeaderEntry:
    def __init__(self, fix_type: FixHeaderType):
        self.fix_type = fix_type

    def start(self):
        ShellDir.goInShellDir()
        batchPodList = BatchPodTools.batchPodList()
        batchPodNames = list(map(lambda x: x.name, batchPodList))
        Print.str("检测到以下模块可执行修复头文件脚本：")
        Print.str(batchPodNames)
        while True:
            targetModuleName = input("请输入需要执行修复头文件脚本的模块名(input ! 退出，all 所有模块执行)：")
            if targetModuleName == "!" or targetModuleName == "！":
                exit(0)
            elif targetModuleName == "all":
                if self.fix_type == FixHeaderType.REPLACE:
                    path = os.getcwd()
                    BatchPodTools.generate_pod_source_files(batchPodList)
                    fix_header = FixHeaderReplaceTool(path, batchPodList)
                    fix_header.find_remote_pod_header()
                for module in batchPodList:
                    Print.stage("开始处理 {0} 模块的头文件".format(module.name))
                    if self.fix_type == FixHeaderType.LINT:
                        self.lint_path(module.path)
                    else:
                        fix_header.find_local_pod_header(module.name)
                        fix_header.replace_import(module.name)
                exit(0)
            else:
                self.module(targetModuleName)

    def module(self, moduleName: str):
        Print.stage("开始处理 {0} 模块的头文件".format(moduleName))
        ShellDir.goInShellDir()
        batchPodList = BatchPodTools.batchPodList()
        batchPodNames = list(map(lambda x: x.name, batchPodList))
        if moduleName in batchPodNames:
            targetModule = list(filter(lambda x: x.name == moduleName, batchPodList))[0]
            if self.fix_type == FixHeaderType.LINT:
                self.lint_path(targetModule.path)
            else:
                path = os.getcwd()
                BatchPodTools.generate_pod_source_files(batchPodList)
                fix_header = FixHeaderReplaceTool(path, batchPodList)
                fix_header.start_fix(moduleName)
                Print.stage("{0} 路径下头文件 replace 成功".format(path))
            exit(0)
        else:
            Print.error(moduleName + "不存在开发列表中")

    def lint_path(self, path: str):
        podpsec_path = ShellDir.findPodspec(path)
        if podpsec_path != None:
            self.__lint_header(podpsec_path)
        else:
            Print.error(path + "路径下不存在podspec文件")

    def __lint_header(self, podspec_path: str):
        lint_header = FixHeaderLintTool()
        lint_header.lint(podspec_path)
        Print.stage("{0} 路径下头文件 lint 成功".format(podspec_path))



import json
import re
from tdf_tools.tools.cmd import Cmd


class PodspecModel:
    def __init__(self, podspec_path: str):
        self.podspec_path = podspec_path

        podspec_json_str: str = Cmd.run("pod ipc spec " + podspec_path)
        self.podspec_json = json.loads(podspec_json_str)

        self.podspec_name = self.__get_name_content__()

        self.source_files = self.__get_source_files__()

        self.podspec_source_root_path = self.__get_source_root_path__()

        if self.podspec_source_root_path == "Classes":
            self.localization_path = self.podspec_source_root_path + "/Localization"
        else:
            self.localization_path = (
                self.podspec_source_root_path + "/Classes/Localization"
            )

        # 本地化特有的 bundle name
        self.podspec_resource_bundle_name = self.__get_strings_resource_bundle_name__()
        self.podspec_resource_bundles = self.__get_resource_bundles__()
        # 自动生成的
        self.auto_string_bundle_source = self.__get_auto_creat_string_bundle_source__()

    # 更新podspec
    def update_pod_spec(self):
        # 更新 dependency 依赖
        self.__check_and_insert_dependency_files()
        # 更新 source_files
        self.__check_and_insert_source_files()
        # 更新 bundle_source
        self.__check_and_insert_auto_string_bundle_source()

    # 获取 podsepc 的text
    def __get_spec_content__(self, path: str) -> str:
        f = open(path)
        content = f.read()
        f.close()
        return content

    def __get_source_files__(self) -> list[str]:
        source_files = self.podspec_json["source_files"]
        if isinstance(source_files, str):
            return [source_files]
        else:
            return source_files

    # 获取 spec 的名字
    def __get_name_content__(self) -> str:
        return self.podspec_json["name"]

    # 获取 source_files 的第一个路径的根路径
    def __get_source_root_path__(self) -> str:
        source_files = self.podspec_json["source_files"]
        first_source_files = self.podspec_name
        if isinstance(source_files, str):
            first_source_files = source_files
        elif isinstance(source_files, list):
            first_source_files = source_files[0]
        return first_source_files.split("/")[0]

    # 本地化特有的 bundle name
    def __get_strings_resource_bundle_name__(self) -> str:
        return self.podspec_name + "Strings"

    # 整个 resource_bundles
    def __get_resource_bundles__(self) -> dict[str:object]:
        try:
            return self.podspec_json["resource_bundles"]
        except:
            return {}

    # 本地化字符串源文件的地址
    def __get_auto_creat_string_bundle_source__(self) -> str:
        return self.podspec_source_root_path + "/{0}_strings/*.lproj".format(
            self.podspec_name
        )

    # 更新 dependency 依赖
    def __check_and_insert_dependency_files(self):
        dependencys: dict = {}
        try:
            dependencys: dict = self.podspec_json["dependencies"]
        except:
            dependencys = {}

        if "TDFInternationalKit" in dependencys.keys():
            print("dependencies中有依赖TDFInternationalKit")
        else:
            insert_str = "  s.dependency 'TDFInternationalKit'"
            f = open(self.podspec_path, "r")
            content = f.read()
            f.close

            # 插入 头文件引用
            pattern = re.compile(r".*?s.dependency.*?[\"']\n")
            results: list[str] = pattern.findall(content)
            if len(results) > 0:
                result: str = results[0]
                newContent = result + insert_str + "\n"
                content = re.sub(pattern, newContent, content, 1)
            else:
                # 插入 s.dependency 到 s.source_files 后面
                pattern = re.compile(r"[\s]?[ ]*[^A-Za-z]s.source_files.*?\n")
                result: str = pattern.findall(content)[0]
                # 处理空格个数
                space_str = result.split("s.")[0]
                print(space_str)
                newContent = result + space_str + insert_str + "\n"
                content = re.sub(pattern, newContent, content, 1)
            # 写入podpspec
            f = open(self.podspec_path, "w")
            f.write(content)
            f.close()

    # 将新增代码的路径添加到 source_files 中
    def __check_and_insert_source_files(self):

        path = self.localization_path + "/**/*.{h,m}"

        # 插入s.source_files
        content = self.__get_spec_content__(self.podspec_path)
        pattern = re.compile(r"[^\n]?[ ]*[^A-Za-z]s.source_files.*?\n")
        result: str = pattern.findall(content)[0]
        result = result.rstrip()
        if path in self.source_files:
            print("s.source_files已包含：" + path)
        else:
            print("s.source_files写入：" + path)
            source_files_last_str = result.split("=")[1]
            source_files = list(map(lambda x: "'{0}'".format(x), self.source_files))
            new_source_files_value = (
                " " + ", ".join(source_files) + ", '" + path + "'\n"
            )
            newContent = result.replace(source_files_last_str, new_source_files_value)
            content = re.sub(pattern, newContent, content, 1)
            print("写入podpspec完成")
            # 写入podpspec
            f = open(self.podspec_path, "w")
            f.write(content)
            f.close()

    # 将新增的本地化资源文件路径加入 resource_bundles 中
    def __check_and_insert_auto_string_bundle_source(self):

        podspec_resource_bundles_list: list[str] = []
        for resourceKey in self.podspec_resource_bundles:
            resource = self.podspec_resource_bundles[resourceKey]
            if isinstance(resource, list):
                podspec_resource_bundles_list.extend(resource)
            elif isinstance(resource, str):
                podspec_resource_bundles_list.append(resource)

        if self.auto_string_bundle_source in podspec_resource_bundles_list:
            print("podspec已经包含" + self.auto_string_bundle_source + "     不需要修改")
        else:
            print("podspec不包含" + self.auto_string_bundle_source)
            print("podspec添加至resource_bundles" + self.auto_string_bundle_source)

            self.podspec_resource_bundles[self.podspec_resource_bundle_name] = [
                self.auto_string_bundle_source
            ]

            resource_bundles_str = " {\n"

            for key in self.podspec_resource_bundles:
                resource_bundles_str += "    '" + key + "' => ["
                resource_bundles_result = self.podspec_resource_bundles[key]
                resource_bundles: list[str] = []
                if isinstance(resource_bundles_result, list):
                    resource_bundles = resource_bundles_result
                elif isinstance(resource_bundles_result, str):
                    resource_bundles = [resource_bundles_result]
                resource_bundles = list(
                    map(lambda x: "'{0}'".format(x), resource_bundles)
                )
                resource_bundles_str += ", ".join(resource_bundles) + "],\n"

            resource_bundles_str += "  }"

            content = self.__get_spec_content__(self.podspec_path)
            pattern = re.compile(r"[^A-Za-z]?s.resource_bundles[^#]*?}\n")
            result = pattern.findall(content)
            if len(result) > 0:
                # 替换
                newContent = " s.resource_bundles =" + resource_bundles_str
                content = re.sub(pattern, newContent, content, 1)
            else:
                # 插入s.resource_bundles 到 s.source_files 后面
                pattern = re.compile(r"[\s]?[ ]*[^A-Za-z]s.source_files.*?\n")
                result: str = pattern.findall(content)[0]
                # 处理空格个数
                space_str = result.split("s.")[0]
                print(space_str)
                newContent = (
                    result
                    + space_str
                    + "s.resource_bundles ="
                    + resource_bundles_str
                    + "\n"
                )
                content = re.sub(pattern, newContent, content, 1)
            # print("podspec添加resource_bundles后",",".join(self.podspec_resource_bundles))
            print("写入podpspec完成")
            # 写入podpspec
            f = open(self.podspec_path, "w")
            f.write(content)
            f.close()



import os
import re
from tdf_tools.tools.translate.ios.tools.batch_pod_tools import (
    BatchPodModel,
)
from tdf_tools.tools.print import Print
from tdf_tools.tools.translate.ios.tools.file_util import FileUtil


class FixHeaderReplaceTool:
    def __init__(self, root_path: str, local_pods: list[BatchPodModel]):
        self.root_path = root_path
        self.pods_path = root_path + "/" + "Pods"
        self.podfile_path = root_path + "/" + "Podfile"
        self.local_module_files: set[str] = set()
        self.local_pods = local_pods
        # retote_header_map 为头文件，value为模块名
        self.remote_header_map: dict[str, str] = {}
        self.local_header_map: dict[str, str] = {}

    def start_fix(self, module_name: str):
        self.find_remote_pod_header()
        self.find_local_pod_header(module_name)
        self.replace_import(module_name)

    # 找出 pod 中远程依赖的文件夹
    def find_remote_pod_header(self):
        Print.title("开始找出 pod 中远程依赖的文件夹")
        self.__find_remote_pod_header(self.pods_path)

    # 找出 pod 中远程依赖的文件夹
    def __find_remote_pod_header(self, path: str):
        files = FileUtil.findAllFile(path)
        for file in files:
            if file.endswith(".h"):
                module_name = file.split(self.pods_path)[1]
                module_name = module_name.split("/")[1]
                if module_name != "Headers":
                    f = file.split("/")[-1]
                    self.remote_header_map[f] = module_name

    # 找出 pod 中本地依赖的文件夹
    def find_local_pod_header(self, module_name: str):
        Print.title("开始找出 pod 中本地依赖的文件夹")
        self.__init_data()
        for pod in self.local_pods:
            for file_path in pod.source_files:
                if pod.name == module_name:
                    # 是当前模块，将模块的所有文件记录下来，存入 module_files 里
                    self.local_module_files.add(file_path)
                else:
                    # 不是本模块的找出头文件对应的模块，生成 header_map
                    if file_path.endswith(".h"):
                        h = file_path.split(r"/")[-1]
                        self.local_header_map[h] = pod.name

    # 替换所有不符合规范的 import
    def replace_import(self, module_name: str):
        Print.title("开始替换所有不符合规范的 import")
        for f in self.local_module_files:
            if os.access(f, os.W_OK):
                self.__find_strings_in_file(f, module_name)
            else:
                Print.error("无可写权限文件：{}".format(f))

    # 初始化数据
    def __init_data(self):
        self.local_module_files = set()
        self.local_header_map = {}

    # 扫描某个文件的字符串
    def __find_strings_in_file(self, filename: str, module_name: str):
        Print.title("开始处理文件：{}\n".format(filename))
        fin = open(filename, "r")
        code = fin.read()
        fin.close()
        new_code = code
        matchs: list[str] = re.findall(r'#import.*"(?:\\.|[^"\n])*"', code)
        for import_str in matchs:
            nameList = re.findall(r"\"(.+?)\"", import_str)
            h_name = nameList[0]
            if "-Swift.h" not in h_name:
                # 先取远端
                f_module_name = self.remote_header_map.get(h_name)
                # 没取到，取本地
                if f_module_name is None:
                    f_module_name = self.local_header_map.get(h_name)
                if f_module_name and f_module_name != module_name:
                    new_imoport = "#import <{}/{}>".format(f_module_name, h_name)
                    new_code = new_code.replace(import_str, new_imoport)
                    Print.warning("替换 {} ==> {}\n".format(import_str, new_imoport))

        if new_code != code:
            fout = open(filename, "w")
            fout.write(new_code)
            fout.close()

        Print.line()



class LocationStringTempClass:
    class_h = r"""
//************************************************************
//
//  代码千万行，规范第一条；编码不规范，队友两行泪！
//
//  Created by pangtouyu on 2022/5/24.
//  
//
// @class TDFLocationStringTempClass
// @abstract 字符串本地化的类，脚本自动生成，不能手动修改
// @discussion 字符串本地化的类，脚本自动生成，不能手动修改
//

#import <Foundation/Foundation.h>

#define TDFLocationStringTempDefine(key, comment) \
        [TDFLocationStringTempClass stringFormStings:(key)]


@interface TDFLocationStringTempClass : NSObject
+ (NSString *)stringFormStings:(NSString *)key;
@end

    """

    class_m = r"""
//
//  TDFLocationStringTempClass.m
//  Test
//
//************************************************************
//*                                                          *
//*            _______     ______         ______             *
//*           |__   __|    |  __ \       / ____/             *
//*             /  /       | (__) |     / /__                *
//*            /__/        |_____/     /_/                   *
//*                                                          *
//*                                                          *
//************************************************************
//
//  代码千万行，规范第一条；编码不规范，队友两行泪！
//
//  Created by pangtouyu on 2022/5/24.
//  
//
// @class TDFLocationStringTempClass
//

#import "TDFLocationStringTempClass.h"
#import <TDFInternationalKit/NSBundle+Language.h>

@implementation TDFLocationStringTempClass

+ (NSString *)stringFormStings:(NSString *)key {
    
    NSString * LocalizableBundlePath = [[NSBundle bundleForClass:NSClassFromString(@"TDFLocationStringTempClass")].resourcePath stringByAppendingString:@"/TDFLocationStringTempBundle.bundle"];
    
    NSString *language = [NSBundle currentLanguage];
    
   
    NSString *path = [[NSBundle bundleWithPath:LocalizableBundlePath] pathForResource:language ofType:@"lproj"];
    NSBundle * LocalizableBundle = [NSBundle bundleWithPath:path] ?:  [NSBundle mainBundle];
    NSString *localizedString = [LocalizableBundle localizedStringForKey:key value:nil table:@"TDFLocationStringTempTable"];

    return localizedString;
}

@end

    """



from abc import ABCMeta, abstractmethod
from enum import Enum
from googletrans import Translator
import requests
import random
from hashlib import md5
from tdf_tools.tools.print import Print


class LanguageType(Enum):
    EN = 1  # 英文
    TH = 2  # 泰语
    ZH = 3  # 简体中文
    CHT = 4  # 繁体中文

    def init(string_type: str):
        if "zh-Hant" in string_type:
            return LanguageType.CHT
        elif "en" in string_type:
            return LanguageType.EN
        elif "th" in string_type:
            return LanguageType.TH
        else:
            return LanguageType.ZH

    def baidu(self) -> str:
        if self.name == LanguageType.EN.name:
            return "en"
        elif self.name == LanguageType.TH.name:
            return "th"
        elif self.name == LanguageType.ZH.name:
            return "zh"
        else:
            return "cht"

    def google(self) -> str:
        if self.name == LanguageType.EN.name:
            return "en"
        elif self.name == LanguageType.TH.name:
            return "th"
        elif self.name == LanguageType.ZH.name:
            return "zh-cn"
        else:
            return "zh-tw"


class TranslateType(Enum):
    BAIDU = 1  # 百度翻译，百度翻译 %d \n 这类符号翻译出来有问题，还是使用谷歌翻译比较好
    GOOGLE = 2  # 谷歌翻译


class Translate(metaclass=ABCMeta):
    @abstractmethod
    def translate(self, text: str, dest=LanguageType.EN, src=LanguageType.ZH) -> str:
        pass


class TranslateTool(Translate):
    def __init__(self, type=TranslateType.BAIDU):
        self.__translator = self.__generate_translate(type)

    def translate(
        self,
        text: str,
        dest=LanguageType.EN,
        src=LanguageType.ZH,
    ) -> str:
        if src == dest:
            return text
        has_double_quotes = False
        # 翻译前去掉 前后双引号
        if text.startswith('"') and text.endswith('"'):
            has_double_quotes = True
            text = text[1:-1]
        if dest == LanguageType.EN or dest == LanguageType.TH:
            # 处理 《》【】“”，因为这些在外语会翻译成双引号，需要加转义符
            text = text.replace("《", r"\"")
            text = text.replace("》", r"\"")
            text = text.replace("【", r"\"")
            text = text.replace("】", r"\"")
            text = text.replace("“", r"\"")
            text = text.replace("”", r"\"")
            text = text.replace("＂", r"\"")
        text = self.__translator.translate(text, src=src, dest=dest)
        # 偶尔会出现 \" 翻译成 \ "，需要检查替换一下
        text = text.replace('\ "', '\\"')
        if has_double_quotes:
            # 翻译完再加上 前后双引号
            text = '"' + text + '"'
        return text

    def __generate_translate(self, type) -> Translate:
        if type == TranslateType.BAIDU:
            return BaiduTranslate()
        else:
            return GoogleTranslate()


class BaiduTranslate(Translate):

    __appid = "20220505001202987"
    __appkey = "qmWBUEi75he1iVZQgqPg"
    __endpoint = "http://api.fanyi.baidu.com"
    __path = "/api/trans/vip/translate"
    __url = __endpoint + __path

    def translate(self, text: str, dest=LanguageType.EN, src=LanguageType.ZH) -> str:
        salt = random.randint(32768, 65536)
        sign = self.__make_md5(self.__appid + text + str(salt) + self.__appkey)

        headers = {"Content-Type": "application/x-www-form-urlencoded"}
        payload = {
            "appid": self.__appid,
            "q": text,
            "from": src.baidu(),
            "to": dest.baidu(),
            "salt": salt,
            "sign": sign,
        }

        r = requests.post(self.__url, params=payload, headers=headers)
        result: dict = r.json()
        error_code = result.get("error_code")
        if error_code != None:
            error_msg = result.get("error_msg")
            Print.error(
                "{0}翻译失败, error_code：{1}，error_msg：{2}".format(
                    text, error_code, error_msg
                )
            )
        trans_result: dict = result["trans_result"]
        return trans_result["dst"]

    def __make_md5(self, s: str, encoding="utf-8"):
        return md5(s.encode(encoding)).hexdigest()


class GoogleTranslate(Translate):
    def translate(self, text: str, dest=LanguageType.EN, src=LanguageType.ZH) -> str:
        return Translator().translate(text, src=src.google(), dest=dest.google()).text



from tdf_tools.tools.shell_dir import ProjectType, ShellDir
from tdf_tools.tools.translate.flutter.flutter_translate_lint import (
    FlutterTranslateLint,
)
from tdf_tools.tools.translate.ios.ios_translate_lint import iOSTranslateLint


class TranslateLint:
    """
    国际化相关：检测源码中是否还有没国际化的文案
    """

    def start(self):
        """
        以交互的方式选择需要 lint 的模块
        """
        ShellDir.dirInvalidate()
        projectType = ShellDir.getProjectType()
        if projectType == ProjectType.FLUTTER:
            FlutterTranslateLint.start()
        elif projectType == ProjectType.IOS:
            iOSTranslateLint.start()

    def module(self, name: str):
        """
        指定模块 lint
        """
        ShellDir.dirInvalidate()
        projectType = ShellDir.getProjectType()
        if projectType == ProjectType.FLUTTER:
            FlutterTranslateLint.module(name)
        elif projectType == ProjectType.IOS:
            iOSTranslateLint.module(name)

    def path(self, path: str):
        """
        指定模块路径 lint，路径为 lib 路径
        """
        projectType = ShellDir.getProjectType()
        if projectType == ProjectType.FLUTTER:
            FlutterTranslateLint.path(path)
        elif projectType == ProjectType.IOS:
            iOSTranslateLint.path(path)



import os
from posixpath import join
import re
from typing import Tuple
from tdf_tools.tools.cmd import Cmd
from tdf_tools.tools.print import Print
from tdf_tools.tools.translate.ios.tools.file_util import FileUtil
from tdf_tools.tools.translate.ios.tools.ios_translate_pattern import (
    iOSTranslatePattern,
)
from tdf_tools.tools.translate.ios.tools.ios_translate_tools import iOSTranslateTools
from tdf_tools.tools.translate.ios.tools.location_string_temp import (
    LocationStringTempClass,
)
from tdf_tools.tools.translate.ios.tools.podspec import PodspecModel


class IosModule:
    # path 组件路径
    def __init__(self, path: str):
        self.path = path
        self.podspecPath = self.__get_podspec_path__()
        if self.podspecPath == "":
            raise ("" + self.path + "未找到podspec文件")
        else:
            self.podspecModel = PodspecModel(self.podspecPath)
            self.auto_string_class_name = (
                self.podspecModel.podspec_name + "AutoLocationString"
            )
            self.auto_string_define_str = (
                self.podspecModel.podspec_name + "LocalizedString"
            )

    def __get_podspec_path__(self) -> str:
        if self.path:
            names = os.listdir(self.path)
            for file in names:
                if file.endswith(".podspec"):
                    return join(self.path, file)
        return ""

    # 检查是否还有未本地化的字符串
    def check_location_string(self):
        # 未本地化的字符串
        un_locating_string_array = iOSTranslateTools.get_all_string(
            self.path + "/" + self.podspecModel.podspec_source_root_path,
            iOSTranslatePattern.pattern_oc__no_localized_chinese_str,
        )
        if len(un_locating_string_array) > 0:
            Print.error(
                "未本地化的字符串{}个: {}",
                format(len(un_locating_string_array), un_locating_string_array),
            )

        # 使用NSLocalizedString本地化的包含中文的字符串
        locating_string_array = iOSTranslateTools.get_all_string(
            self.path + "/" + self.podspecModel.podspec_source_root_path,
            iOSTranslatePattern.pattern_oc__nslocalized_str,
        )
        if len(locating_string_array) > 0:
            Print.error(
                "使用NSLocalizedString本地化的包含中文的字符串本地化的字符串{}个: {}locating_string_array".format(
                    len(locating_string_array), locating_string_array
                )
            )

        # 使用 TDFModuleLocalizedString 本地化，但是 .strings 文件中没有翻译的文件
        module_locating_string_array = iOSTranslateTools.get_all_string(
            self.path + "/" + self.podspecModel.podspec_source_root_path,
            '(?<={}\(@")[^"]*?[\u4E00-\u9FA5][^"]*?(?=".*?\))'.format(
                self.auto_string_define_str
            ),
        )
        module_locating_string_dic = {}
        for string in module_locating_string_array:
            module_locating_string_dic[string] = string

        for string_type in FileUtil.LOCAL_STRING_TYPES:
            dic_from_strings = iOSTranslateTools.get_module_dict(
                self.path + "/" + self.podspecModel.podspec_source_root_path,
                string_type,
            )
            for key in dic_from_strings.keys():
                del module_locating_string_dic[key]

            un_translate_strings = module_locating_string_dic.keys()
            if len(un_translate_strings) > 0:
                Print.error(
                    "{}.string中使用{}本地化的包含中文的字符串本地化的字符串{}个: {}".format(
                        string_type,
                        self.auto_string_define_str,
                        len(un_translate_strings),
                        un_translate_strings,
                    ),
                )

    # 检查 .strings 文件的有效性
    def check_strings_file(self):
        Print.stage("检查 .strings 文件的有效性")
        podspec = PodspecModel(self.podspecPath)
        podspec_resource_bundle = self.path + "/" + podspec.auto_string_bundle_source
        podspec_resource_bundle = podspec_resource_bundle.replace("/*.lproj", "")
        for root, dirs, files in os.walk(podspec_resource_bundle):
            for file in files:
                if file.endswith(".strings"):
                    file_path = root + "/" + file
                    result = Cmd.system("plutil -lint " + root + "/" + file)
                    if result != 0:
                        Print.error(file_path + "文件有错误")
        Print.stage("检查 .strings 文件的有效")

    def change_location_string(self):
        print("检查并修改未本地化的字符串")

        def __replace_impl__(str):
            print(
                "替换" + str + "\n" + self.auto_string_define_str + "(" + str + ", nil)"
            )
            return self.auto_string_define_str + "(" + str + ", nil)"

        def __replace_nslocalized_impl__(str: str):
            print(
                "替换"
                + str
                + "\n"
                + str.replace("NSLocalizedString", self.auto_string_define_str)
            )
            pattern = re.compile(r"%s" % (iOSTranslatePattern.patten_localized_suffix))
            results = pattern.findall(str)
            if len(results) > 0:
                result = results[0]
                # 把 NSLocalizedString 的第二个参数，全部换成 nil
                str = str.replace(result, ", nil)")

            return str.replace("NSLocalizedString", self.auto_string_define_str)

        path_result = []

        # 替换使用 NSLocalizedString 修饰的字符串
        path = self.path + "/" + self.podspecModel.podspec_source_root_path
        path_list = iOSTranslateTools.change_all_string(
            path,
            iOSTranslatePattern.pattern_oc__nslocalized_str,
            __replace_nslocalized_impl__,
        )
        print(path_list)
        path_result.extend(path_list)

        # 未国际化的字符串
        path = self.path + "/" + self.podspecModel.podspec_source_root_path
        path_list = iOSTranslateTools.change_all_string(
            path,
            iOSTranslatePattern.pattern_oc__no_localized_chinese_str,
            __replace_impl__,
        )
        print(path_list)
        path_result.extend(path_list)

        path_result = list(set(path_result))

        # 增加头文件引用
        for path_str in path_result:
            self.remove_static_str(path_str)
            self.add_import_str(path_str)

    # oc  static 不支持本地化
    def remove_static_str(self, path: str):
        # 查询 static 的字符串声明
        f = open(path, "r")
        content = f.read()
        f.close
        old_content = content
        pattern = re.compile(
            r"static .*?" + self.auto_string_define_str + "[\s\S]*?\);\n"
        )

        def __remove_localized_str__(matched):
            result = matched.group()
            result = result.replace(self.auto_string_define_str + "(", "")
            result = result.replace(", nil)", "")
            return result

        content = re.sub(pattern, __remove_localized_str__, content)
        # 写入
        if old_content != content:
            f = open(path, "w")
            f.write(content)
            f.close()

    def add_import_str(self, path: str):
        insert_str = '#import "' + self.auto_string_class_name + '.h"'
        f = open(path, "r")
        content = f.read()
        f.close
        if self.auto_string_define_str in content == False:
            return

        if insert_str in content:
            return
        # 插入 头文件引用
        pattern = re.compile(r".*?#import.*?\n")
        result: str = pattern.findall(content)[0]
        newContent = result + insert_str + "\n"
        content = re.sub(pattern, newContent, content, 1)
        # 写入podpspec
        f = open(path, "w")
        f.write(content)
        f.close()

    # 检查 podsepc，修改 podsepc
    def checkout_change_pod_spec(self):
        print("开始检查podspec")
        print("模块podspec路径:" + self.podspecModel.podspec_path)
        print("模块名称:" + self.podspecModel.podspec_name)
        print(
            "模块resource_bundles name:" + self.podspecModel.podspec_resource_bundle_name
        )
        print("模块当前resource_bundles path:", self.podspecModel.podspec_resource_bundles)
        print("模块当前source_files:", self.podspecModel.source_files)
        # 检查并更新podspec文件
        self.podspecModel.update_pod_spec()

    # 生成本地化文件
    def create_update_location_files(self) -> dict[str, dict[str, str]]:
        # 生成本地化文件
        # 支持的语言类型
        local_string_string_types = list(
            map(lambda x: x + ".lproj", FileUtil.LOCAL_STRING_TYPES)
        )
        # 生成文件夹路径
        local_string_name = self.podspecModel.podspec_name + ".strings"
        local_string_file_paths: list[(str, str)] = []
        for string_type in local_string_string_types:
            file_string_path = (
                self.path
                + "/"
                + self.podspecModel.auto_string_bundle_source.replace(
                    "*.lproj", string_type
                )
            )
            local_string_file_paths.append((file_string_path, string_type))
        # 创建文件夹和文件
        for (path, string_type) in local_string_file_paths:
            string_file = path + "/" + local_string_name
            if os.path.exists(path):
                print("已存在文件夹：" + path)
            else:
                print("不存在文件夹：" + path + "\n创建文件:" + path)
                os.makedirs(
                    path,
                )
            if os.path.exists(string_file):
                print("已存在文件：" + string_file)
            else:
                print("不存在文件：" + string_file + "\n创建文件:" + string_file)
                file = open(string_file, "a+")
                file.close

            # 同步组件其他文件复制到此文件
            def __strings_filter__(path: str):
                if string_type in path:
                    if string_file != path:
                        return True
                return False

        # 获取需要翻译的所有key
        pattern = (
            "(?<=" + self.auto_string_define_str + '\(@")[\s\S]*?(?="[ ]*,[ ]*nil)'
        )
        locating_string_array = iOSTranslateTools.get_all_string(
            self.path + "/" + self.podspecModel.podspec_source_root_path,
            pattern,
        )

        print(locating_string_array)
        print("本地化的key共%d个" % (len(locating_string_array)))
        locating_truple_arrary = []
        for locating_str in locating_string_array:
            locating_truple_arrary.append((locating_str, ""))

        # 新增的翻译
        new_translate_dict: dict[str, dict[str, str]] = {}

        for (path, string_type) in local_string_file_paths:
            string_file = path + "/" + local_string_name
            file_type = string_type.split(".lproj")[0]
            new_dict = iOSTranslateTools.update_iOS_string_key_Values(
                (string_file, string_type),
                locating_truple_arrary,
                self.path + "/" + self.podspecModel.podspec_source_root_path,
            )
            new_translate_dict[file_type] = new_dict

        return new_translate_dict

    # 添加本地化需要的OC文件并加入podspec的 sourcefile
    def create_update_oc_files(self):
        creat_file_full_path = self.path + "/" + self.podspecModel.localization_path
        # 添加文件夹
        if os.path.exists(creat_file_full_path):
            print("已存在文件夹：" + creat_file_full_path)
        else:
            print("不存在文件夹：" + creat_file_full_path + "\n创建文件:" + creat_file_full_path)
            os.makedirs(creat_file_full_path)

        for file_content in [
            LocationStringTempClass.class_h,
            LocationStringTempClass.class_m,
        ]:
            string_file = creat_file_full_path + "/" + self.auto_string_class_name
            if file_content == LocationStringTempClass.class_h:
                string_file = string_file + ".h"
            else:
                string_file = string_file + ".m"

            if os.path.exists(string_file):
                print("已存在文件：" + string_file)
            else:
                print("不存在文件：" + string_file + "\n创建文件:" + string_file)
                # 替换占位符
                file = open(string_file, "a+")

                file_content = file_content.replace(
                    "TDFLocationStringTempClass", self.auto_string_class_name
                )
                file_content = file_content.replace(
                    "TDFLocationStringTempBundle",
                    self.podspecModel.podspec_resource_bundle_name,
                )
                file_content = file_content.replace(
                    "TDFLocationStringTempTable", self.podspecModel.podspec_name
                )
                file_content = file_content.replace(
                    "TDFLocationStringTempDefine", self.auto_string_define_str
                )

                file.write(file_content)
                file.close



class iOSTranslatePattern:

    # search文件名称后缀
    ios_suffix = [".m"]
    ios_special_str_list = ["%@", '\\"']

    # 正则表达式
    patten_localized_suffix = ',[ ]*@"[\s\S]*?"\)'
    # ObjectC 包含中文的字符串，未本地化
    pattern_oc__no_localized_chinese_str = '(?<!\()@"[^"]*?[\u4E00-\u9FA5][^"]*?"'
    # ObjectC 包含中文的字符串，未本地化  特殊，不处理的情况  例如static
    pattern_oc__static_str = "static [\s\S]*?;\n"
    # ObjectC 包含中文的字符串，未本地化  特殊，不处理的情况  例如NSString * const
    pattern_oc__const_str = "NSString[ ]*\*[ ]*const[\s\S]*?;\n"
    # ObjectC 包含中文的字符串 并使用 NSLocalizedString本地化
    pattern_oc__nslocalized_str = (
        'NSLocalizedString\(@"[^"]*?[\u4E00-\u9FA5][^"]*?".*?\)'
    )
    # 过滤掉埋点相关代码的正则
    pattern_filter_track_str = "\[TDFAnalyticsSDK.share[\s\S]*?\];"
    # 过滤掉路由相关代码的正则
    pattern_filter_router_str = '@"tdf-manager:[^"]*?[\u4E00-\u9FA5][^"]*?"|@"tdf-flutter:[^"]*?[\u4E00-\u9FA5][^"]*?"|@"tdf-ccd:[^"]*?[\u4E00-\u9FA5][^"]*?"|@"YunCash:[^"]*?[\u4E00-\u9FA5][^"]*?"'



import sys
from tdf_tools.pipelines.module import Module
from tdf_tools.pipelines.translate import Translate
from tdf_tools.pipelines.router import Router
from tdf_tools.pipelines.fix_header import FixHeader
from tdf_tools.pipelines.package import Package

from tdf_tools.pipelines.git import Git
from tdf_tools.pipelines.upgrade import Upgrade


class Pipeline:
    """
    二维火 Flutter 脚手架工具，包含项目构建，依赖分析，git等功能。。
    """

    def __init__(self):
        self.module = Module()
        self.translate = Translate()
        self.package = Package()
        self.router = Router()
        self.fixHeader = FixHeader()

    def upgrade(self):
        """
        tdf_tools upgrade：升级插件到最新版本
        """
        Upgrade().run()

    def git(self, *kwargs, **kwargs1):
        """
        tdf_tools git【git 命令】：批量操作 git 命令, 例如 tdf_tools git push
        """
        Git(sys.argv[2:]).run()



from tdf_tools.tools.shell_dir import ProjectType, ShellDir
from tdf_tools.tools.print import Print
from tdf_tools.tools.fix_header.fix_header_entry import FixHeaderEntry, FixHeaderType


class FixHeaderLint:
    """
    iOS修复头文件lint相关：tl fixHeader lint -h 查看详情
    """

    def __init__(self):
        self.__fix_entry = FixHeaderEntry(FixHeaderType.LINT)

    def start(self):
        """
        tl fixHeader lint start，会以交互式进行进行头文件lint
        """
        ShellDir.dirInvalidate()
        projectType = ShellDir.getProjectType()
        if projectType == ProjectType.FLUTTER:
            Print.error("tl fixHeader 只支持iOS")
        elif projectType == ProjectType.IOS:
            self.__fix_entry.start()

    def module(self, name: str):
        """
        tl fixHeader lint module【模块名】，lint 指定模块名
        """
        ShellDir.dirInvalidate()
        projectType = ShellDir.getProjectType()
        if projectType == ProjectType.FLUTTER:
            Print.error("tl fixHeader 只支持iOS")
        elif projectType == ProjectType.IOS:
            self.__fix_entry.module(name)

    def path(self, path: str):
        """
        tl fixHeader lint path【模块路径】，指定模块路径进行lint
        """
        self.__fix_entry.lint_path(path)
        exit(0)



import os
from io import TextIOWrapper
import re
from tdf_tools.tools.print import Print
from tdf_tools.tools.translate.ios.tools.batch_pod_tools import (
    BatchPodTools,
)


class FixHeaderLintTool:
    __files = []
    __h_files = []
    __fail_logs = []
    __ex_swift = "-Swift.h"  # 不处理swift文件

    def lint(self, podspec_path: str):
        file_list: set[str] = BatchPodTools.get_source_files(podspec_path)
        self.__find_all_files(file_list)
        self.__find_wrong_import(file_list)

    # 查找文件，并归类
    def __find_all_files(self, files: set[str]):
        for f in files:
            file_name = f.split(r"/")[-1]
            if file_name.endswith(self.__ex_swift):
                continue
            if file_name.endswith(".h"):
                self.__h_files.append(file_name)
            self.__files.append(file_name)

    # 找到所有不符合规范的 import
    def __find_wrong_import(self, file_paths: set[str]):
        for f in file_paths:
            if os.access(f, os.R_OK):
                self.__lint_strings_in_file(f)
            else:
                Print.title("无可读权限文件：{0}".format(f))
        if self.__fail_logs:
            for str in self.__fail_logs:
                Print.warning(str)
            Print.error("以上文件有头文件引用错误，请修改")

    # 扫描某个文件的字符串，找到不符合规范的 import
    def __lint_strings_in_file(self, filename):
        if os.path.isdir(filename):
            Print.error("asdadsa")
        Print.title("开始 lint 文件：{}\n".format(filename))
        fin: TextIOWrapper = open(filename, "r")
        code = fin.read()
        fin.close()
        matchs: list[str] = re.findall(r'\n#import.*"(?:\\.|[^"\n])*"', code)
        for import_str in matchs:
            h_name = import_str.replace(" ", "").split('#import"')[1].split('"')[0]
            if self.__ex_swift not in h_name:
                if h_name not in self.__h_files:
                    self.__fail_logs.append("{}===需要修改{}的引用".format(filename, h_name))



# !/usr/bin/env python
# -*- coding: UTF-8 -*-

# 依赖分析【有向无环图，有向有环图】


import json
from mimetypes import init
import os
from ruamel import yaml
from tdf_tools.tools.print import Print
from tdf_tools.tools.shell_dir import ShellDir
from tdf_tools.tools.module.module_tools import ModuleTools


class DependencyNode:
    def __init__(self):
        self.nodeName = ""
        self.parent = []  # 父亲节点列表
        self.children = []  # 子孙节点列表
        self.isShell = False  # 是否是shell模块
        self.modulePath = ""  # 模块路径
        self.delete = False


class DependencyAnalysis:
    def __init__(self) -> None:
        self.__initJsonData__ = None
        self.__moduleNameList__ = None
        self.__shellName__ = None

    @property
    def __initJsonData(self):
        if self.__initJsonData__ is None:
            self.__initJsonData__ = ModuleTools.getInitJsonData()
        return self.__initJsonData__

    @property
    def __moduleNameList(self):
        if self.__moduleNameList__ is None:
            self.__moduleNameList__ = ModuleTools.getModuleNameList()
        return self.__moduleNameList__

    @property
    def __shellName(self):
        if self.__shellName__ is None:
            self.__shellName__ = self.__initJsonData["shellName"]
        return self.__shellName__

    # 分析lock文件，获取所有的packages
    def _analysisLock(self):
        # 读取lock内容
        with open("pubspec.lock", encoding="utf-8") as f:
            doc = yaml.round_trip_load(f)
            if isinstance(doc, dict) and doc.__contains__("packages"):
                f.close()
                return doc["packages"]

    # 生成依赖图
    def _generateDependenciesMap(self):
        for package in self.__moduleDependenciesMap:
            for module in self.__moduleNameList:
                if package == module:
                    # 到这一步表明当前这个模块属于开发模块且在当前模块的依赖模块列表中，是当前模块的子模块
                    self._mNodeDict[self.__moduleName].children.append(package)
                    self._mNodeDict[package].parent.append(self.__moduleName)

    # 返回二维数组，用于并发打tag
    def _generateDependenciesOrder(self):
        resList = []
        while self._existNode():
            itemList = []

            for item in self._mNodeDict:
                node = self._mNodeDict[item]
                if isinstance(node, DependencyNode):
                    if not node.delete:
                        if len(node.children) == 0:
                            itemList.append(node.nodeName)
                            node.delete = True

            deleteNodeList = []
            for item in self._mNodeDict:
                node = self._mNodeDict[item]
                if isinstance(node, DependencyNode):
                    if node.delete:
                        deleteNodeList.append(node.nodeName)

            for item in self._mNodeDict:
                node = self._mNodeDict[item]
                if isinstance(node, DependencyNode):
                    for deleteItem in deleteNodeList:
                        if node.children.__contains__(deleteItem):
                            node.children.remove(deleteItem)

            if len(itemList) == 0:
                break
            resList.append(itemList)
        return json.dumps(resList)

    # 返回一维数组，用于从下至上执行upgrade
    def _generateDependenciesOrderForUpgrade(self):
        resList = []
        while self._existNode():
            for item in self._mNodeDict:
                node = self._mNodeDict[item]
                if isinstance(node, DependencyNode):
                    if not node.delete:
                        if len(node.children) == 0:
                            resList.append(node.nodeName)
                            node.delete = True

            deleteNodeList = []
            for item in self._mNodeDict:
                node = self._mNodeDict[item]
                if isinstance(node, DependencyNode):
                    if node.delete:
                        deleteNodeList.append(node.nodeName)

            for item in self._mNodeDict:
                node = self._mNodeDict[item]
                if isinstance(node, DependencyNode):
                    for deleteItem in deleteNodeList:
                        if node.children.__contains__(deleteItem):
                            node.children.remove(deleteItem)

        print(resList)

        return resList

    def _existNode(self):
        for item in self._mNodeDict:
            node = self._mNodeDict[item]
            if isinstance(node, DependencyNode) and not node.delete:
                return True
        return False

    def _subChildCount(self, childName):
        for item in self._mNodeDict:
            node = self._mNodeDict[item]
            if isinstance(node, DependencyNode):
                if not node.delete:
                    if node.children.__contains__(childName):
                        node.children.remove(childName)

    def generate(self):

        self._mNodeDict = dict()
        # 初始化子模块的节点列表
        for module in self.__moduleNameList:
            node = DependencyNode()
            node.nodeName = module
            ShellDir.goInTdfFlutterDir()
            os.chdir(module)
            node.modulePath = os.getcwd()
            self._mNodeDict[module] = node
        # 初始化壳节点
        # node = DependencyNode()
        # node.nodeName = self.__shellName
        # node.isShell = True
        # ShellDir.goInShellDir()
        # node.modulePath = os.getcwd()
        # self._mNodeDict[node.nodeName] = node

        # 读取子模块lock文件
        for module in self.__moduleNameList:
            self.__moduleName = module
            # self.__moduleGenPath = projectModuleDir + "/" + module
            os.chdir(self._mNodeDict[module].modulePath)
            self.__moduleDependenciesMap = self._analysisLock()
            self._generateDependenciesMap()
        # 读取壳模块lock文件
        # self.__moduleName = self.__shellName
        # os.chdir(self._mNodeDict[self.__shellName].modulePath)
        # self.__moduleDependenciesMap = self._analysisLock()
        # self._generateDependenciesMap()

        return self._generateDependenciesOrder()

    def getDependencyOrder(self):

        self._mNodeDict = dict()
        # 初始化子模块的节点列表
        for module in self.__moduleNameList:
            node = DependencyNode()
            node.nodeName = module
            ShellDir.goInTdfFlutterDir()
            os.chdir(module)
            node.modulePath = os.getcwd()
            self._mNodeDict[module] = node
        # 初始化壳节点
        node = DependencyNode()
        node.nodeName = self.__shellName
        node.isShell = True
        ShellDir.goInShellDir()
        node.modulePath = os.getcwd()
        self._mNodeDict[node.nodeName] = node

        # 读取子模块lock文件
        for module in self.__moduleNameList:
            self.__moduleName = module
            # self.__moduleGenPath = projectModuleDir + "/" + module
            os.chdir(self._mNodeDict[module].modulePath)
            self.__moduleDependenciesMap = self._analysisLock()
            self._generateDependenciesMap()
        # 读取壳模块lock文件
        self.__moduleName = self.__shellName
        os.chdir(self._mNodeDict[self.__shellName].modulePath)
        self.__moduleDependenciesMap = self._analysisLock()
        self._generateDependenciesMap()

        return self._generateDependenciesOrderForUpgrade()



#!/usr/bin/env python
# encoding=utf-8

import json
import os
import subprocess
import copy
from ruamel import yaml
import requests
from tdf_tools.tools.dependencies_analysis import DependencyAnalysis
from tdf_tools.tools.config.initial_json_config import InitialJsonConfig
from tdf_tools.tools.shell_dir import ShellDir

PRIVATE_REPOSITORY_HOST = "http://10.1.85.141"

# 版本号自增 依赖修改

# 读取当前文件夹内的pubspec.yaml文件


def yamlLoad():
    # 读取pubsepc.yaml内容
    with open("pubspec.yaml", encoding="utf-8") as f:
        doc = yaml.round_trip_load(f)
        if isinstance(doc, dict):
            f.close()
            return doc


# 规范版本号


def strictVersion(version):
    if isinstance(version, str):
        tagList = version.split(".")
        if len(tagList) == 3 and int(tagList[1]) <= 9:
            if len(tagList[2]) == 2:
                return version
            elif len(tagList[2]) == 1:
                return version + "1"
            else:
                print("❌ tag号规范检测失败，请严格按照规范修改tag号,例：x.y.z, （0<=x, 0<=y<=9, 00<=z<=99）")
                exit(-1)
        else:
            print("❌ tag号规范检测失败，请严格按照规范修改tag号,例：x.y.z, （0<=x, 0<=y<=9, 00<=z<=99）")
            exit(-1)


# 获取已经发布过的远程仓库列表


def getRemoteRepositories():
    try:
        r = requests.get(PRIVATE_REPOSITORY_HOST + "/webapi/packages?size=1000")
        mDict = json.loads(r.text)
        mRepositoryMap = {}
        for item in mDict["data"]["packages"]:
            mRepositoryMap[item["name"]] = item["latest"]
        return mRepositoryMap
    except:
        print("❌ 读取私库内模块列表失败，请确保私库正常运行")
        exit(-1)


# 版本号自增, tagNumber = 1019


def addVersion(tagNumber, shouldAdd=True):
    try:
        if shouldAdd:
            tagNumber = tagNumber + 1
        upgradeTag = "{0}.{1}.{2}".format(
            int(tagNumber / 1000),
            int((tagNumber % 1000) / 100),
            "{:0>2d}".format(tagNumber % 100),
        )
        # print("对{0}进行自增,自增后为{1}".format(tagNumber - 1, upgradeTag))
        return upgradeTag
    except Exception as ex:
        print("❌ 版本号自增失败:{0}".format(ex))
        exit(-1)


# 版本号比对


def getUpgradeVersion(currentVersion, remoteVersion):
    try:
        currentVersionList = currentVersion.split(".")
        currentVersionNumber = (
            int(currentVersionList[0]) * 1000
            + int(currentVersionList[1]) * 100
            + int(currentVersionList[2])
        )

        # 如果远端版本为空，则直接对当前版本进行自增操作即可
        if remoteVersion == None:
            return addVersion(currentVersionNumber)

        remoteVersionList = remoteVersion.split(".")
        remoteVersionNumber = (
            int(remoteVersionList[0]) * 1000
            + int(remoteVersionList[1]) * 100
            + int(remoteVersionList[2])
        )

        if currentVersionNumber > remoteVersionNumber:
            return addVersion(currentVersionNumber, shouldAdd=False)

        if currentVersionNumber > remoteVersionNumber:
            return addVersion(currentVersionNumber)
        else:
            return addVersion(remoteVersionNumber)
    except Exception as ex:
        print("❌ 版本号比对出现异常:{0}".format(ex))
        exit(-1)


def tagInfoObj():

    analysis = DependencyAnalysis()
    treeJson = json.loads(analysis.generate())
    # print(treeJson)

    remoteRepositoryMap = getRemoteRepositories()
    initJsonData = InitialJsonConfig()

    result = {}

    for item in initJsonData.moduleNameList:
        ShellDir.goInTdfFlutterDir()
        os.chdir(item)
        # print("检测模块：{0}".format(item))

        yamlInfo = yamlLoad()
        if yamlInfo.__contains__("version") is not True:
            print("❌模块{0}yaml文件中没有检测到version版本号字段，请设置".format(item))
            exit(-1)

        current = strictVersion(yamlInfo["version"])

        remote = None
        if remoteRepositoryMap.__contains__(item):
            remote = remoteRepositoryMap[item]

        result[item] = {
            "remote": remote,
            "current": current,
            "upgrade": getUpgradeVersion(current, remote),
            "sort": 0,
        }

    for item in result:
        for index in range(len(treeJson)):
            if treeJson[index].__contains__(item):
                result[item]["sort"] = index

    print(json.dumps(result))
    exit(0)


# 修改依赖
def changeVersionAndDependencies(jsonData):

    # 例："{\"tdf_presell\":{\"remote\":\"1.0.05\",\"current\":\"1.0.04\",\"upgrade\":\"1.0.06\",\"sort\":1},\"tdf_goods_videos_manager\":{\"remote\":\"1.0.06\",\"current\":\"1.0.05\",\"upgrade\":\"1.0.07\",\"sort\":1},\"tdf_widgets\":{\"remote\":\"1.0.12\",\"current\":\"1.0.12\",\"upgrade\":\"1.0.13\",\"sort\":0}}"
    # tempStr = "{\"tdf_presell\":{\"remote\":\"1.0.05\",\"current\":\"1.0.04\",\"upgrade\":\"1.0.06\",\"sort\":1},\"tdf_goods_videos_manager\":{\"remote\":\"1.0.06\",\"current\":\"1.0.05\",\"upgrade\":\"1.0.07\",\"sort\":1},\"tdf_widgets\":{\"remote\":\"1.0.12\",\"current\":\"1.0.12\",\"upgrade\":\"1.0.13\",\"sort\":0}}"
    # jsonData = json.loads(tempStr)

    initJsonData = InitialJsonConfig()
    for item in initJsonData.moduleNameList:
        ShellDir.goInTdfFlutterDir()
        os.chdir(item)
        # print("\n修改模块版本号和依赖：{0}".format(item))

        yamlInfo = yamlLoad()
        # dependency_overrides依赖
        dependencyOverridesDict = dict
        # dependencies依赖
        dependencyDict = dict
        if yamlInfo.__contains__("dependency_overrides"):
            dependencyOverridesDict = copy.deepcopy(yamlInfo["dependency_overrides"])

        if yamlInfo.__contains__("dependencies"):
            dependencyDict = copy.deepcopy(yamlInfo["dependencies"])

        # dependencyOverridesDict不为空，才需要修改dependencyDict
        # 修改规则：1.dependency中没有，dependency_overrides中有，则添加为tdg依赖方式
        #         2.dependency中有，dependency_overrides中有，则覆盖为tag依赖方式
        if (
            dependencyOverridesDict is not None
            and isinstance(dependencyOverridesDict, dict)
            and dependencyOverridesDict.__len__() > 0
        ):
            for dependencyOverridesItem in dependencyOverridesDict:
                upwardReference = True
                if dependencyOverridesItem in dependencyDict:
                    if (
                        isinstance(dependencyDict[dependencyOverridesItem], dict)
                        and "version" in dependencyDict[dependencyOverridesItem]
                    ):
                        itemDict = dependencyDict[dependencyOverridesItem]
                        if isinstance(itemDict, dict) and "version" in itemDict:
                            versionStr = itemDict["version"]
                            if isinstance(versionStr, str) and versionStr.__contains__(
                                "^"
                            ):
                                upwardReference = True

                # print("--- 修改节点{0}的依赖 如下".format(dependencyOverridesItem))

                dependencyDict[dependencyOverridesItem] = {
                    "hosted": {
                        "name": "{0}".format(dependencyOverridesItem),
                        "url": "http://10.1.85.141",
                    },
                    "version": "{0}{1}".format(
                        "^" if upwardReference else "", jsonData[item]["upgrade"]
                    ),
                }
                # 输出修改后的节点信息
                print(json.dumps(dependencyDict[dependencyOverridesItem]))

            # 重写
            try:
                yamlInfo["version"] = jsonData[item]["upgrade"]
                del yamlInfo["dependency_overrides"]
                yamlInfo["dependencies"] = dependencyDict

                with open("pubspec.yaml", "w+", encoding="utf-8") as reW:
                    yaml.round_trip_dump(
                        yamlInfo,
                        reW,
                        default_flow_style=False,
                        encoding="utf-8",
                        allow_unicode=True,
                    )
                    reW.close()
            except Exception as e:
                print("重写时出现异常，错误原因：{0}".format(e))
                exit(-1)


def createAndPushTag(tagModuleList):
    initJsonData = InitialJsonConfig()

    for moduleName in tagModuleList:

        isExist = False
        for item in initJsonData.moduleNameList:
            if moduleName == item:
                isExist = True
                ShellDir.goInTdfFlutterDir()
                os.chdir(item)
                # print("\n{0}模块开始执行tag操作...".format(item))

                # 在yaml中获version，作为tag
                yamlInfo = yamlLoad()
                tag = yamlInfo["version"]
                # print("获取tag号：{0}".format(tag))

                # print("exec: git tag {0}".format(tag))
                res = os.popen("git tag {0}".format(tag)).read()
                # print(res)

                # print("exec: git push origin {0}".format(tag))
                res = os.popen("git push origin {0}".format(tag)).read()
                # print(res)

        if isExist is False:
            print("模块{0}没有找到".format())
            exit(-1)



# -*- coding: UTF-8 -*-

from ast import Str
import re
import base64

from tdf_tools.tools.translate.ios.tools.ios_translate_pattern import (
    iOSTranslatePattern,
)

simple_decode_key = "dsfsf7w9e76f87ew6f87ew6f8we"


class StringUtil:
    def simpleEncode(data: str) -> str:
        # base编码
        data = (simple_decode_key + data).encode()
        res = base64.b64encode(data)
        res = bytes.decode(res)
        return res

    def simpleDecode(data: str) -> str:
        # base64解码
        # print('解码前'+data)
        res = base64.b64decode(data)
        res = bytes.decode(res)
        res = res.replace(simple_decode_key, "")
        # print('解码后'+res)

        return res

    def get_chinese_string(
        text: str, pattern_str: str, spical_strs: list[str], except_list=[]
    ) -> list[str]:
        for except_str in except_list:
            text = re.sub(except_str, "", text)

        text = StringUtil.get_replaced_str(text, spical_strs)
        pattern = re.compile(r"%s" % (pattern_str))
        result = pattern.findall(text)

        newResult: list[str] = []
        for item in result:
            if len(item) > 0:
                str = StringUtil.get_origin_str(item, spical_strs)
                newResult.append(str)

        return newResult

    def get_changed_chinese_string(
        text: str, pattern_str: str, spical_strs: list[str], repl
    ) -> str:

        pattern = re.compile(r"%s" % (iOSTranslatePattern.pattern_filter_track_str))
        track_result = pattern.findall(text)
        spical_strs.extend(track_result)

        pattern = re.compile(r"%s" % (iOSTranslatePattern.pattern_filter_router_str))
        router_result = pattern.findall(text)
        spical_strs.extend(router_result)

        text = StringUtil.get_replaced_str(text, spical_strs)
        pattern = re.compile(r"%s" % (pattern_str))

        def new_repl(matched):
            return repl(StringUtil.get_origin_str(matched.group(), spical_strs))

        result = re.sub(pattern, new_repl, text)
        result = StringUtil.get_origin_str(result, spical_strs)
        return result

    # 特殊的字符会先被替换，然后正则匹配，最后替换回来，减少正则复杂度，提高效率
    # 获取特殊字符加密后的字符串
    def get_replaced_str(oldStr: str, spical_strs: list[str]) -> str:
        for text in spical_strs:
            if len(text) > 0:
                oldStr = oldStr.replace(text, StringUtil.simpleEncode(text))

        return oldStr

    # 获取特殊字符被解密后的字符串
    def get_origin_str(oldStr: str, spical_strs: list[str]) -> str:
        for text in spical_strs:
            if len(text) > 0:
                oldStr = oldStr.replace(
                    StringUtil.get_replaced_str(text, spical_strs), text
                )

        return oldStr



from tdf_tools.tools.print import Print
from tdf_tools.tools.shell_dir import ShellDir
from tdf_tools.tools.translate.ios.tools.batch_pod_tools import (
    BatchPodTools,
)
from tdf_tools.tools.translate.ios.tools.ios_module import IosModule


class iOSTranslateLint:

    # 交互式 国际化
    def start():
        ShellDir.goInShellDir()
        batchPodList = BatchPodTools.batchPodList()
        batchPodNames = list(map(lambda x: x.name, batchPodList))

        Print.str("检测到以下模块可执行国际化 lint 脚本：")
        Print.str(batchPodNames)
        while True:
            targetModule = input("请输入需要执行国际化 lint 脚本的模块名(input ! 退出，all 所有模块执行)：")
            if targetModule == "!" or targetModule == "！":
                exit(0)
            elif targetModule == "all":
                for module in batchPodList:
                    iOSTranslateLint.module(module.name)
                exit(0)
            else:
                iOSTranslateLint.module(targetModule)
                exit(0)

    # 指定 模块国际化
    def module(name: str):
        ShellDir.goInShellDir()
        batchPodList = BatchPodTools.batchPodList()
        batchPodNames = list(map(lambda x: x.name, batchPodList))
        if name in batchPodNames:
            Print.title(name + " 模块国际化 lint 脚本开始执行")
            pod = list(filter(lambda x: x.name == name, batchPodList))[0]
            iOSTranslateLint.path(pod.path)
            Print.title(name + " 模块国际化 lint 执行完成")
        else:
            Print.error(name + " 模块不在开发列表中")

    # 指定路径国际化
    def path(path_str: str):
        module = IosModule(path_str)
        # 检查 .strings 文件的有效性
        module.check_strings_file()
        # 检查是否还有未本地化的字符串
        module.check_location_string()



import json
import os
import re
from tdf_tools.tools.cmd import Cmd
from tdf_tools.tools.print import Print
from tdf_tools.tools.translate.ios.tools.batch_pod_tools import BatchPodModel
from tdf_tools.tools.translate.ios.tools.file_util import FileUtil
from tdf_tools.tools.translate.ios.tools.ios_translate_tools import iOSTranslateTools


class iOSTranslateIntegrate:

    # 整合 json 文件
    def integrate_json_root(self) -> dict[str, dict[str, str]]:
        all_dict: dict[str, str] = {}
        for type in FileUtil.LOCAL_STRING_TYPES:
            path = FileUtil.localizable_path() + "/" + type + "/localizable.json"
            strings_dict = iOSTranslateTools.get_dic_from_json_file(path)
            strings_dict = self.__delete_double_quotes(strings_dict)
            all_dict[type] = strings_dict
        return all_dict

    # 整合主工程的 .strins 文件
    def integrate_strins_root(self, path) -> dict[str, dict[str, str]]:
        all_dict: dict[str, str] = {}
        for type in FileUtil.LOCAL_STRING_TYPES:
            path = path + type + ".lproj" + "/Localizable.strings"
            strings_dict = iOSTranslateTools.get_dic_from_string_file(path)
            strings_dict = self.__delete_double_quotes(strings_dict)
            all_dict[type] = strings_dict
        return all_dict

    # 整合模块
    def integrate_pod(self, pod: BatchPodModel) -> dict[str, dict[str, str]]:
        all_dict: dict[str, dict[str, str]] = {}
        for root, dirs, files in os.walk(pod.path):
            for file in files:
                if file.endswith(pod.name + ".strings"):
                    file_path = root + "/" + file
                    pattern = re.compile(r"[^/]*(?=\.lproj/)")
                    results = pattern.findall(file_path)
                    if len(results) > 0:
                        type = results[0]
                        strings_dict = iOSTranslateTools.get_dic_from_string_file(
                            file_path
                        )
                        strings_dict = self.__delete_double_quotes(strings_dict)
                        all_dict[type] = strings_dict
        return all_dict

    # 新增翻译到 中心化的json中，并上传
    def new_dict_and_upload(self, new_dict: dict[str, dict[str, str]]):
        Print.stage("新增翻译到 中心化的json中，并上传")
        # 目前的json字典
        old_json_dict = self.integrate_json_root()
        new_json_dict = self.merge_dict(new_dict, old_json_dict)
        self.write_dict_to_json(new_json_dict)
        self.upload_json()
        Print.stage("新增翻译到 中心化的json中，并上传成功")

    # 合并两个翻译文件，two会覆盖one
    def merge_dict(
        self, one: dict[str, dict[str, str]], two: dict[str, dict[str, str]]
    ) -> dict[str, dict[str, str]]:
        all_dict: dict[str, str] = {}
        for type in FileUtil.LOCAL_STRING_TYPES:
            one_type_dict = one[type]
            two_type_dict = two[type]
            one_type_dict.update(two_type_dict)
            all_dict[type] = one_type_dict
        return all_dict

    # 写入翻译内容到 json 中
    def write_dict_to_json(self, json_dict: dict[str, dict[str, str]]):
        for type in FileUtil.LOCAL_STRING_TYPES:
            json_path = FileUtil.localizable_path() + "/" + type
            json_file_path = json_path + "/localizable.json"
            if os.path.exists(json_path):
                print("已存在文件夹：" + json_path)
            else:
                print("不存在文件夹：" + json_path + "\n创建文件:" + json_path)
                os.makedirs(
                    json_path,
                )
            if os.path.exists(json_file_path):
                print("已存在文件：" + json_file_path)
            else:
                print("不存在文件：" + json_file_path + "\n创建文件:" + json_file_path)
                file = open(json_file_path, "a+")
                file.close()
            type_json_dict = json_dict[type]
            json_str = json.dumps(
                type_json_dict, sort_keys=True, ensure_ascii=False, indent=4
            )
            json_file = open(json_file_path, "w+")
            json_file.write(json_str)
            json_file.close()

    # 上传 jons 到远端
    def upload_json(self):
        localizable_path = FileUtil.localizable_path()
        Cmd.runAndPrint("git -C {0} pull origin master".format(localizable_path))
        Cmd.runAndPrint("git -C {0} add .".format(localizable_path))
        Cmd.runAndPrint("git -C {0} commit -m 'new translate'".format(localizable_path))
        Cmd.runAndPrint("git -C {0} push origin master".format(localizable_path))

    # 删除字段里面字符串前后的双引号
    def __delete_double_quotes(self, target: dict[str, str]) -> dict[str, str]:
        new_dict = {}
        for k, v in target.items():
            # 翻译前去掉 前后双引号
            if k.startswith('"') and k.endswith('"'):
                k = k[1:-1]
            if v.startswith('"') and v.endswith('"'):
                v = v[1:-1]
            new_dict[k] = v
        return new_dict



from tdf_tools.tools.print import Print
from tdf_tools.tools.shell_dir import ShellDir
from tdf_tools.tools.translate.ios.tools.batch_pod_tools import (
    BatchPodModel,
    BatchPodTools,
)
from tdf_tools.tools.translate.ios.tools.ios_module import IosModule
from tdf_tools.tools.translate.ios.tools.ios_translate_integrate import (
    iOSTranslateIntegrate,
)


class iOSTranslate:

    # 交互式 国际化
    def translate(self):
        ShellDir.goInShellDir()
        batchPodList = BatchPodTools.batchPodList()
        batchPodNames = list(map(lambda x: x.name, batchPodList))

        Print.str("检测到以下模块可执行国际化脚本：")
        Print.str(batchPodNames)
        while True:
            targetModule = input("请输入需要执行国际化脚本的模块名(input ! 退出，all 所有模块执行)：")
            if targetModule == "!" or targetModule == "！":
                exit(0)
            elif targetModule == "all":
                for module in batchPodList:
                    self.translate_module(module.name)
                exit(0)
            else:
                self.translate_module(targetModule)
                exit(0)

    # 指定 模块国际化
    def translate_module(self, name: str):
        ShellDir.goInShellDir()
        batchPodList = BatchPodTools.batchPodList()
        batchPodNames = list(map(lambda x: x.name, batchPodList))
        if name in batchPodNames:
            Print.title(name + " 模块国际化脚本开始执行")
            pod = list(filter(lambda x: x.name == name, batchPodList))[0]
            self.__generate_translate(pod)
            Print.title(name + " 模块国际化执行完成")
        else:
            Print.error(name + " 模块不在开发列表中")

    # 整合各个模块翻译文件到一起
    def integrate(self):

        self.integrate: iOSTranslateIntegrate = iOSTranslateIntegrate()
        ShellDir.goInShellDir()
        batchPodList = BatchPodTools.batchPodList()
        batchPodNames = list(map(lambda x: x.name, batchPodList))

        Print.str("检测到以下模块可整合翻译文件：")
        Print.str(batchPodNames)

        while True:
            targetModule = input("请输入需要执行整合的模块名(input ! 退出，all 所有模块执行)：")
            all_dict = self.integrate.integrate_json_root()
            if targetModule == "!" or targetModule == "！":
                exit(0)
            elif targetModule == "all":
                all_dict = self.integrate.integrate_json_root()
                for module in batchPodList:
                    module_dict = self.__integrate_module(module.name)
                    all_dict = self.integrate.merge_dict(module_dict, all_dict)
            else:
                module_dict = self.__integrate_module(targetModule)
                all_dict = self.integrate.merge_dict(module_dict, all_dict)
            self.integrate.write_dict_to_json(all_dict)
            self.integrate.upload_json()
            exit(0)

    # 指定 模块国际化
    def __integrate_module(self, name: str) -> dict[str, dict[str, str]]:
        ShellDir.goInShellDir()
        batchPodList = BatchPodTools.batchPodList()
        batchPodNames = list(map(lambda x: x.name, batchPodList))
        if name in batchPodNames:
            Print.title(name + " 模块整合开始执行")
            pod = list(filter(lambda x: x.name == name, batchPodList))[0]
            Print.title(name + " 模块整合完成")
            return self.integrate.integrate_pod(pod)
        else:
            Print.error(name + " 模块不在开发列表中")

    # 翻译入口
    def __generate_translate(self, target: BatchPodModel):
        module = IosModule(target.path)
        # 检查 podsepc，修改 podsepc
        module.checkout_change_pod_spec()
        # 创建本地化相关 oc 类文件
        module.create_update_oc_files()
        # 检查并修改oc文件中未本地化的字符串
        module.change_location_string()
        # 检查是否还有未本地化的字符串
        module.check_location_string()
        # 生成本地化文件
        new_translate_dict = module.create_update_location_files()
        # 检查生成的 strings 文件格式是否正确
        module.check_strings_file()
        # 新增翻译到中心化json中，并上传到远程
        iOSTranslateIntegrate().new_dict_and_upload(new_translate_dict)



# coding=utf-8
import json
from tdf_tools.tools.print import Print
from tdf_tools.tools.translate.ios.tools.file_util import FileUtil
from tdf_tools.tools.translate.ios.tools.ios_translate_pattern import (
    iOSTranslatePattern,
)
from tdf_tools.tools.translate.ios.tools.string_util import StringUtil
from tdf_tools.tools.translate.tools.translate_tool import (
    LanguageType,
    TranslateTool,
    TranslateType,
)
from io import TextIOWrapper


class iOSTranslateTools:
    def get_all_string(dir: str, pattern: str) -> list[str]:
        total_list = []
        for path in FileUtil.get_allfile(dir, iOSTranslatePattern.ios_suffix):
            # 打开一个文件
            f: TextIOWrapper = open(path)
            chinese_list = StringUtil.get_chinese_string(
                f.read(),
                pattern,
                spical_strs=iOSTranslatePattern.ios_special_str_list,
                except_list=[
                    # iOSTranslatePattern.pattern_oc__static_str,
                    # iOSTranslatePattern.pattern_oc__const_str,
                    iOSTranslatePattern.pattern_filter_track_str,
                    iOSTranslatePattern.pattern_filter_router_str,
                ],
            )
            f.close()
            if len(chinese_list) > 0:
                # print("文件名: "+ path)
                for item in chinese_list:
                    total_list.append(item)
        # 去重后返回
        return list(set(total_list))

    # repl 需要实现一个方法去替换 匹配的方法 返回修改的文件数组
    def change_all_string(dir: str, pattern: str, repl) -> list[str]:
        path_list = []
        for path in FileUtil.get_allfile(dir, iOSTranslatePattern.ios_suffix):
            # 打开一个文件
            f = open(path, "r")
            old_text = f.read()
            result = StringUtil.get_changed_chinese_string(
                old_text, pattern, iOSTranslatePattern.ios_special_str_list, repl
            )
            f.close()
            if old_text != result:
                path_list.append(path)
                f = open(path, "w")
                f.write(result)
                f.close()
        return path_list

    # 向strings文件加入key value  truple (path,string_type)   如果存在则不处理 入参为list  元素为key value的元组     source_path为strings源文件目录 优先从此文件翻译
    def update_iOS_string_key_Values(args, list, source_path) -> dict[str, str]:
        path = args[0]
        string_type = args[1]
        dic_from_strings = iOSTranslateTools.get_dic_with_dir(source_path, string_type)
        string_no_trans_count = 0
        # 已经国际化过的数量
        string__local_trans_count = 0
        # 需要谷歌翻译的数量
        string__goole_trans_count = 0
        # 新增的翻译文案
        string__goole_trans_dict: dict[str, str] = {}
        # 翻译错误的数量
        string__trans_error_count = 0

        f = open(path)
        orig_content = f.read()
        # 判断最后一个是否是回车
        content = orig_content.strip() + "\n"
        f.close

        content_dict = {}
        for (key, value) in list:
            if (key in content) == False:
                # 未本地化
                string_no_trans_count = string_no_trans_count + 1
                if len(value) == 0:
                    value = iOSTranslateTools.__translate_str_from_local(
                        key, dic_from_strings
                    )
                    if len(value) > 0:
                        string__local_trans_count = string__local_trans_count + 1
                    else:
                        value = iOSTranslateTools.__translate_str_from_goole(
                            key, string_type
                        )
                        if len(value) > 0:
                            string__goole_trans_count = string__goole_trans_count + 1
                            string__goole_trans_dict[key] = value
                        else:
                            string__trans_error_count = string__trans_error_count + 1
                            continue

                content_dict[key] = value

        for key in sorted(content_dict.keys()):
            content = content + '"' + key + '" = "' + content_dict[key] + '";\n'

        # 写入
        Print.title(
            string_type
            + "共"
            + str(len(list))
            + "个包含中文的字符串,"
            + str(string_no_trans_count)
            + "个未翻译,"
            + str(string__local_trans_count)
            + "个本地翻译,"
            + str(string__goole_trans_count)
            + "个谷歌翻译,"
            + str(string__trans_error_count)
            + "个翻译失败"
        )
        if orig_content != content:
            f = open(path, "w")
            f.write(content)
            f.close()

        return string__goole_trans_dict

    def __translate_str_from_local(str: str, dic_local: dict):
        result = dic_local.get(str, "")
        return result

    def __translate_str_from_goole(str: str, string_type) -> str:
        translator = TranslateTool(TranslateType.GOOGLE)
        languageType = LanguageType.init(string_type)
        return translator.translate(str, dest=languageType)

    # 获取组件中 .strings 中的所有键值对
    def get_module_dict(source_dir: str, string_type: str) -> dict:
        result = {}
        # 获取组件下所有的国际化键值对
        for path in FileUtil.get_allfile(source_dir, ".strings"):
            if string_type in path:
                result.update(iOSTranslateTools.get_dic_from_string_file(path))
        return result

    # 获取组件、全局json中的所有键值对
    def get_dic_with_dir(source_dir: str, string_type: str) -> dict:
        if len(source_dir) == 0:
            return {}
        result = {}

        app_strings_path = FileUtil.localizable_path()

        # 获取组件下所有的国际化键值对
        module_dict = iOSTranslateTools.get_module_dict(source_dir, string_type)
        result.update(module_dict)

        # 获取全部国际化键值对
        for path in FileUtil.get_allfile(app_strings_path, ".json"):
            type_vaule = string_type.split(".lproj")[0]
            if type_vaule in path:
                result.update(iOSTranslateTools.get_dic_from_json_file(path))

        return result

    # 获取 .strings 中的国际化键值对
    def get_dic_from_string_file(file_path) -> dict:
        with open(file_path, "r") as load_f:
            dic = {}
            for line in load_f:
                line = line.replace(";\n", "")
                v = line.split("=")
                count = len(v)
                # 分割不正常
                if count % 2 == 1:
                    continue
                value = ""
                key = ""
                for index, val in enumerate(v):
                    if index < count / 2:
                        key = key + val.strip()
                    else:
                        value = value + val.strip()

                # 去掉前后双引号
                if key.startswith('"') and key.endswith('"'):
                    key = key[1:-1]
                if value.startswith('"') and value.endswith('"'):
                    value = value[1:-1]
                dic[key] = value
            return dic

    # 获取 .json 中的国际化键值对
    def get_dic_from_json_file(file_path) -> dict:
        with open(file_path, "r", encoding="utf-8") as rf:
            jsonData = json.loads(rf.read())
            rf.close()
            return jsonData



from genericpath import isdir
import os
from posixpath import join
from tdf_tools.tools.cmd import Cmd


class FileUtil:

    LOCALOZABLE_PATH = "/.tdf_tools/localizable"
    TOOLS_PATH = "/.tdf_tools/tools"
    LOCAL_STRING_TYPES = [
        "en",
        "zh-Hans",
        "zh-Hant",
        "th",
    ]

    def localizable_path() -> str:
        tdf_tools_path = os.path.expanduser("~") + "/.tdf_tools"
        localizable_path = os.path.expanduser("~") + FileUtil.LOCALOZABLE_PATH
        if not os.path.exists(tdf_tools_path):
            os.mkdir(tdf_tools_path)
        if os.path.exists(localizable_path):
            Cmd.run("git -C {} pull".format(localizable_path))
        else:
            Cmd.run(
                "git -C {} clone git@git.2dfire.net:app/other/localizable.git".format(
                    tdf_tools_path
                )
            )
        return localizable_path

    # 创建并返回 get_files.rb 文件
    def generate_get_files_rb() -> str:
        path = os.path.expanduser("~") + FileUtil.TOOLS_PATH
        if not os.path.exists(path):
            os.makedirs(path)

        ruby_code = r"""
        for path in ARGV
            paths = Dir::glob(path)
            puts paths
        end"""
        get_files = path + "/get_files.rb"
        file = open(get_files, "w")
        file.write(ruby_code)
        file.close()
        return get_files

    def get_allfile(path, suffix_str_list=[]) -> list[str]:
        if isdir(path) == False:
            print("不存在路径：" + path)
            return []
        all_file = []
        for (root, dirs, files) in os.walk(path):
            for f in files:
                for suffix_str in suffix_str_list:
                    if f.endswith(suffix_str):
                        all_file.append(join(root, f))
        return all_file

    def findAllFile(path):
        for root, ds, fs in os.walk(path):
            for f in fs:
                fullname = os.path.join(root, f)
                yield fullname
